"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Execute = exports.executePromptMessages = void 0;
const core_1 = require("@oclif/core");
const errors_1 = require("@oclif/core/lib/errors");
const file_loaders_1 = require("../helpers/file-loaders");
const execute_templates_request_1 = require("../http/execute-templates-request");
const get_connections_request_1 = require("../http/get-connections-request");
const filter_connections_by_template_1 = require("../prompt-data/filter-connections-by-template");
const get_candidate_fixtures_1 = require("../prompt-data/get-candidate-fixtures");
const get_template_files_1 = require("../prompt-data/get-template-files");
const get_external_data_files_1 = require("../prompt-data/get-external-data-files");
const render_prompt_1 = require("../prompt-data/render-prompt");
const render_repeat_input_1 = require("../rendering/render-repeat-input");
const render_template_result_1 = require("../rendering/render-template-result");
const path_1 = require("../customer/path");
const config_1 = require("../customer/config");
const template_source_code_1 = require("../customer/template-source-code");
const ux_action_1 = require("../helpers/ux-action");
exports.executePromptMessages = {
    template: "Select a template to execute",
    fixture: "Select a catalog fixture to run the template against",
    externalData: "Select external data to run the template against",
    connection: "Select a connection to execute templates against",
};
class Execute extends core_1.Command {
    static flags = {
        "template-path": core_1.Flags.string({
            description: "The path to the template to execute. Must be in the 'src/templates' directory.",
        }),
        "fixture-path": core_1.Flags.string({
            description: "The path to the fixture to execute the template against. Must be in the 'src/fixtures' directory.",
        }),
        "external-data-path": core_1.Flags.string({
            description: "The path to the external data to execute the template against. Must be in the 'src/fixtures/external_data' directory.",
        }),
        "connection-id": core_1.Flags.string({
            description: "The ID of the connection to execute the template against.",
        }),
    };
    static description = "Execute a template against a connection and fixture to see the resulting transformed data. Each value not provided as a flag will be prompted for when the command is executed.";
    static examples = [
        "$ <%= config.bin %> execute",
        "$ <%= config.bin %> execute --template-path=item/item.jsonata",
        "$ <%= config.bin %> execute --template-path=variation/variation.jsonata",
        "$ <%= config.bin %> execute --template-path=item_group/item_group.jsonata --fixture-path=item_group.json",
        "$ <%= config.bin %> execute --template-path=grouping/grouping.jsonata --connection-id=example-connection-id",
        "$ <%= config.bin %> execute --template-path=mapping/mapping.jsonata",
    ];
    input = {};
    async run() {
        let result = null;
        try {
            const templateInput = await this.getTemplateExecInput();
            const config = await (0, ux_action_1.uxAction)("📡 Reading your config", async () => {
                return await (0, config_1.getRepositoryConfigFile)();
            })();
            core_1.ux.log("\n");
            result = await (0, execute_templates_request_1.executeTemplates)({
                template: (0, file_loaders_1.getJSONataTemplate)(templateInput.templatePath, "utf-8").toString(),
                helpers: (await (0, template_source_code_1.getHelpersSourceCode)(config.helpers)) ?? "",
                targetData: (0, file_loaders_1.getCatalogFixture)(templateInput.fixturePath),
                externalData: templateInput.externalDataPath
                    ? (0, file_loaders_1.getExternalDataFixture)(templateInput.externalDataPath)
                    : {},
                connectionId: templateInput.connectionId,
            });
            core_1.ux.log("\n");
            (0, render_template_result_1.renderTemplateResult)(result);
            if (!result?.success) {
                this.exit(1);
            }
        }
        finally {
            (0, render_repeat_input_1.renderRepeatInput)(this, this.input);
        }
    }
    async parseFlags() {
        const { flags } = await this.parse(Execute);
        if (flags["template-path"]) {
            this.input["template-path"] = (0, path_1.cleanupTemplatePath)(flags["template-path"]);
        }
        if (flags["fixture-path"]) {
            this.input["fixture-path"] = (0, path_1.cleanupFixturePath)(flags["fixture-path"]);
        }
        if (flags["external-data-path"]) {
            this.input["external-data-path"] = (0, path_1.cleanupExternalDataPath)(flags["external-data-path"]);
        }
        if (flags["connection-id"]) {
            this.input["connection-id"] = flags["connection-id"];
        }
    }
    async getTemplateExecInput() {
        const allConnections = await (0, get_connections_request_1.getConnectionsForCompany)();
        await this.parseFlags();
        if (!this.input["template-path"]) {
            const choices = await (0, get_template_files_1.getTemplateFiles)();
            this.input["template-path"] = await (0, render_prompt_1.renderPrompt)({
                choices,
                promptMessage: exports.executePromptMessages.template,
                emptyMessage: "No templates found",
            });
        }
        if (!this.input["fixture-path"]) {
            this.input["fixture-path"] = await (0, render_prompt_1.renderPrompt)({
                choices: (0, get_candidate_fixtures_1.getCandidateFixtures)(this.input["template-path"]),
                promptMessage: exports.executePromptMessages.fixture,
                emptyMessage: "No fixtures found for this template",
            });
        }
        if (!this.input["external-data-path"]) {
            const choices = (0, get_external_data_files_1.getExternalDataFiles)();
            if (choices.length) {
                this.input["external-data-path"] = await (0, render_prompt_1.renderPrompt)({
                    choices,
                    promptMessage: exports.executePromptMessages.externalData,
                    emptyMessage: "No external data found",
                });
            }
        }
        const connectionId = await this.calculateConnectionId(allConnections, this.input["template-path"]);
        return {
            templatePath: this.input["template-path"],
            fixturePath: this.input["fixture-path"],
            externalDataPath: this.input["external-data-path"] ?? "",
            connectionId,
        };
    }
    async calculateConnectionId(allConnections, templatePath) {
        let connection = allConnections.find((c) => c.id === this.input["connection-id"]);
        if (this.input["connection-id"] && !connection) {
            throw new errors_1.CLIError(`Connection with ID ${this.input["connection-id"]} not found`, {
                suggestions: ["Check that the connectionId is correct"],
            });
        }
        if (!connection) {
            connection = await this.getConnectionForTemplate(templatePath, allConnections);
            this.input["connection-id"] = connection.id;
        }
        return connection.id;
    }
    async getConnectionForTemplate(templatePath, allConnections) {
        const choices = await (0, filter_connections_by_template_1.filterConnectionsByTemplate)(templatePath, allConnections);
        return await (0, render_prompt_1.renderPrompt)({
            choices,
            promptMessage: exports.executePromptMessages.connection,
            emptyMessage: "No mapped connections found for this template",
        });
    }
}
exports.Execute = Execute;
